#    Copyright 2013 Chris Caron
#
#    This is free software: you can redistribute it and/or modify it under the
#    terms of the GNU General Public License as published by the Free Software
#    Foundation, either version 3 of the License, or (at your option) any later
#    version.
#
#    This file is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
#    more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this file. If not, see http://www.gnu.org/licenses/.
###############################################################################
# SPEC File Author: Chris Caron <lead2gold@gmail.com>
# Date: Apr 25th, 2014
# Version: 2.8.10-1
#
# Description:
#
#  With respect to the blog post here:
#    http://nuxref.wordpress.com/2013/10/05/gimp-v2-8-on-centos-6/
#  This is the SPEC file created to allow CentOS 6.x or Red Hat 6.x
#  to run GIMP v2.8 in a parallel environment and not obstruct the 
#  existing environment.
#
# Disable Debug Info Packaging
%define  debug_package %{nil}

# disable check-buildroot
# Sadly this is nessisary because we're breaking rules already installing
# glib 3.4 on Centos 6 along with GTK, GDK, etc.
%define __arch_install_post %{nil}

# lcms:        lcms support
%bcond_without lcms
# python:      python support
%bcond_without python
# mp:          multi processor support
%bcond_without mp
# print:       build the print plugin (if you don't build it externally)
%bcond_without print
# gutenprint:  require gutenprint-plugin (instead of gimp-print-plugin) if
#              internal print plugin isn't built
%bcond_without gutenprint
# convenience: install convenience symlinks
%bcond_without convenience
# gudev:       use gudev to discover special input devices
%if ! 0%{?fedora}%{?rhel} || 0%{?fedora} >= 15 || 0%{?rhel} >= 6
# use gudev from F-15/RHEL7 on
%bcond_without gudev
%else
%bcond_with gudev
%endif
# aalib:       build with AAlib (ASCII art gfx library)
%if 0%{?rhel}
# don't use aalib on RHEL
%bcond_with aalib
%else
%bcond_without aalib
%endif
# hardening:   use various compiler/linker flags to harden binaries against
#              certain types of exploits
%bcond_without hardening
# poppler:     use from version 0.20 on which is GPLv2/GPLv3 dual-licensed
%if 0%{?fedora}%{?rhel} == 0 || 0%{?fedora} >= 18 || 0%{?rhel} >= 6
%bcond_without poppler
%else
%bcond_with poppler
%endif
# helpbrowser  build webkit-based help browser
%if 0%{?rhel}
%bcond_with helpbrowser
%else
%bcond_without helpbrowser
%endif
# hardcode python interpreter in python plug-ins
%bcond_without hardcoded_python

%define install_root /opt/gimp28

Summary: GIMP v2.8 for Centos 6
License: GPLv3+ and GPLv3
Group: Applications/Multimedia
Name: gimp28
Prefix: /usr
Release: 1%{?dist}.nuxref

%global gettext_version 20
%global lib_api_version 2.0
%global binver 2.8

Version: 2.8.10
Source0: ftp://ftp.gimp.org/pub/gimp/v2.8/gimp-2.8.10.tar.bz2
Source1: http://ftp.acc.umu.se/pub/GNOME/sources/glib/2.34/glib-2.34.0.tar.xz
Source2: ftp://ftp.gimp.org/pub/babl/0.1/babl-0.1.10.tar.bz2
Source3: ftp://ftp.gimp.org/pub/gegl/0.2/gegl-0.2.0.tar.bz2
Source4: http://download.gnome.org/sources/atk/2.2/atk-2.2.0.tar.bz2
Source5: http://cairographics.org/releases/cairo-1.12.16.tar.xz
Source6: http://ftp.gnome.org/pub/gnome/sources/pango/1.29/pango-1.29.4.tar.bz2
Source7: http://ftp.gnome.org/pub/gnome/sources/gdk-pixbuf/2.24/gdk-pixbuf-2.24.1.tar.xz
Source8: http://ftp.gnome.org/pub/gnome/sources/gtk+/2.24/gtk+-2.24.10.tar.xz
Source9: gimp28.desktop.tgz

Packager: Chris Caron <lead2gold@gmail.com>
URL: http://www.gimp.org/
Buildroot: %{_tmppath}/%{name}-%{version}-root

Requires: aalib
Requires: freetype
Requires: fontconfig
Requires: hicolor-icon-theme
Requires: xdg-utils
Requires: pygtk2

Provides: gimp28

BuildRequires: aalib, aalib-devel, libexif-devel, libjpeg-devel, libpng-devel
BuildRequires: libtiff-devel, libmng-devel, libXpm-devel, librsvg2-devel
BuildRequires: libffi-devel, libwmf-devel, gettext, intltool
BuildRequires: webkitgtk-devel, python-devel, zlib-devel, pygtk2-devel
BuildRequires: chrpath, freetype-devel, ghostscript-devel, iso-codes-devel
BuildRequires: bzip2-devel, curl-devel fontconfig-devel, gnome-keyring-devel
BuildRequires: libgnomeui-devel, python-devel, zlib-devel, libX11-devel
BuildRequires: sed, findutils
%ifarch x86_64
%define _libdir %{_exec_prefix}/lib64
%else
%define _libdir %{_exec_prefix}/lib
%endif

%description
GIMP (GNU Image Manipulation Program) is a powerful image composition and
editing program, which can be extremely useful for creating logos and other
graphics for webpages. GIMP has many of the tools and filters you would expect
to find in similar commercial offerings, and some interesting extras as well.
GIMP provides a large image manipulation toolbox, including channel operations
and layers, effects, sub-pixel imaging and anti-aliasing, and conversions, all
with multi-level undo.

%prep
%setup -q -c -a 0
%setup -q -T -D -a 1
%setup -q -T -D -a 2
%setup -q -T -D -a 3
%setup -q -T -D -a 4
%setup -q -T -D -a 5
%setup -q -T -D -a 6
%setup -q -T -D -a 7
%setup -q -T -D -a 8
%setup -q -T -D -a 9

%build
rm -fr %{buildroot} &>/dev/null || true
mkdir -p %{buildroot}

# Update Environment Vairables
export LIBRARY_PATH=%{buildroot}%{_libdir}
export LD_RUN_PATH=%{install_root}%{_libdir}
export LD_LIBRARY_PATH=%{buildroot}%{_libdir}

# Compilation Flags
export CFLAGS="$CFLAGS -fPIC -I%{buildroot}/usr/include %optflags"
export CXXFLAGS="$CXXFLAGS -fPIC -I%{buildroot}/usr/include %optflags"
export LDFLAGS="$LDFLAGS -pie -Wl,-rpath=%{install_root}%{_libdir} -L$LD_LIBRARY_PATH"
export PKG_CONFIG_PATH=$LD_LIBRARY_PATH/pkgconfig

reset() {
   # A Simple function we use to
   _LIBPATH=$1
   [ -z "$_LIBPATH" ] && _LIBPATH=$LD_LIBRARY_PATH
   # Fix Paths
   find %{buildroot}%{_libdir}/pkgconfig \
         -name *.pc \
         -exec sed -i \
            -e "s|^includedir=.*|includedir=%{buildroot}/usr/include|g" {} \;
   find %{buildroot}%{_libdir}/pkgconfig \
         -name *.pc \
         -exec sed -i \
            -e "s|^libdir=.*|libdir=$_LIBPATH|g" {} \;
}

pushd glib-2.34.0
%configure \
    CFLAGS="$CFLAGS -fno-strict-aliasing" \
    CXXFLAGS="$CXXFLAGS -fno-strict-aliasing"
#%makeinstall
make %{?_smp_mflags}
make DESTDIR=%{buildroot} install
popd

reset
pushd cairo-1.12.16
%configure \
    CFLAGS="$CFLAGS -fno-strict-aliasing" \
    CXXFLAGS="$CXXFLAGS -fno-strict-aliasing"
#%makeinstall
make %{?_smp_mflags}
make DESTDIR=%{buildroot} install
popd

reset
pushd babl-0.1.10
%configure \
    CFLAGS="$CFLAGS -fno-strict-aliasing" \
    CXXFLAGS="$CXXFLAGS -fno-strict-aliasing"
#%makeinstall
make %{?_smp_mflags}
make DESTDIR=%{buildroot} install
popd

reset
pushd gegl-0.2.0
%configure \
    CFLAGS="$CFLAGS -fno-strict-aliasing" \
    CXXFLAGS="$CXXFLAGS -fno-strict-aliasing"
%makeinstall
# make %{?_smp_mflags}
#make DESTDIR=%{buildroot} install
popd

reset
pushd atk-2.2.0
%configure \
    CFLAGS="$CFLAGS -fno-strict-aliasing" \
    CXXFLAGS="$CXXFLAGS -fno-strict-aliasing"
%makeinstall
# make %{?_smp_mflags}
#make DESTDIR=%{buildroot} install
popd

reset
pushd pango-1.29.4
%configure \
    CFLAGS="$CFLAGS -fno-strict-aliasing" \
    CXXFLAGS="$CXXFLAGS -fno-strict-aliasing" \
    --enable-gtk-doc \
    --enable-doc-cross-references \
   --with-included-modules=basic-fc
%makeinstall
# make %{?_smp_mflags}
#make DESTDIR=%{buildroot} install
popd

reset
pushd gdk-pixbuf-2.24.1
%configure
    CFLAGS="$CFLAGS -fno-strict-aliasing" \
    CXXFLAGS="$CXXFLAGS -fno-strict-aliasing"
#%makeinstall
make %{?_smp_mflags}
make DESTDIR=%{buildroot} RUN_QUERY_LOADER_TEST=false install
touch %{buildroot}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders.cache

(cd %{buildroot}/usr/bin
 mv gdk-pixbuf-query-loaders gdk-pixbuf-query-loaders-%{__isa_bits}
)
popd

reset
pushd gtk+-2.24.10
%configure
    CFLAGS="$CFLAGS -fno-strict-aliasing" \
    CXXFLAGS="$CXXFLAGS -fno-strict-aliasing"
%makeinstall
# make %{?_smp_mflags}
#make DESTDIR=%{buildroot} install
popd

reset
pushd gimp-%{version}
%configure \
    CFLAGS="$CFLAGS -fno-strict-aliasing" \
    CXXFLAGS="$CXXFLAGS -fno-strict-aliasing" \
%if %{with python}
    --enable-python \
%else
    --disable-python \
%endif
%if %{with mp}
    --enable-mp \
%else
    --disable-mp \
%endif
%if %{with print}
    --with-print \
%else
    --without-print \
%endif
%if %{with lcms}
    --with-lcms=lcms2 \
%else
    --without-lcms \
%endif
    --enable-gimp-console \
%if %{with aalib}
    --with-aa \
%else
    --without-aa \
%endif
%if %{with gudev}
    --with-gudev --without-hal \
%else
    --with-hal --without-gudev \
%endif
%ifos linux
    --with-linux-input \
%endif
%if use_poppler
    --with-poppler \
%else
    --without-poppler \
%endif
%if %{with helpbrowser}
    --with-webkit \
%else
    --without-webkit \
%endif
    --with-libtiff --with-libjpeg --with-libpng --with-libmng --with-libjasper \
    --with-libexif --with-librsvg --with-libxpm --with-gvfs --with-alsa \
    --with-dbus --with-script-fu --with-cairo-pdf \
    --enable-shared --disable-static
make %{?_smp_mflags}
make DESTDIR=%{buildroot} install
popd

# One Final Reset
reset %{install_root}$LD_LIBRARY_PATH

# We now have a nicely prepared %{buildroot} directory to
# work with; we need to move it back here
[ -d buildroot ] && rm -rf buildroot
mv %{buildroot} buildroot

%install
rm -rf %{buildroot}
mv buildroot %{buildroot}

# update rpaths
find %{buildroot} -type f -print0 | xargs -0 -L 20 chrpath \
      --replace "%{install_root}%{_libdir}" --keepgoing 2>/dev/null || :

# remove .la files
find %{buildroot} -name \*.la -exec %__rm -f {} \;

# install convenience symlinks
ln -snf gimp-%{binver} %{buildroot}%{_bindir}/gimp
ln -snf gimp-%{binver}.1 %{buildroot}%{_mandir}/man1/gimp.1
ln -snf gimp-console-%{binver} %{buildroot}/%{_bindir}/gimp-console
ln -snf gimp-console-%{binver}.1 %{buildroot}/%{_mandir}/man1/gimp-console.1
ln -snf gimptool-%{lib_api_version} %{buildroot}%{_bindir}/gimptool
ln -snf gimptool-%{lib_api_version}.1 %{buildroot}%{_mandir}/man1/gimptool.1
ln -snf gimprc-%{binver}.5 %{buildroot}/%{_mandir}/man5/gimprc.5

# Hardcode python interpreter in shipped python plug-ins. This actually has no
# effect because gimp maps hashbangs with and without the /usr/bin/env detour
# to the system python interpreter, but this will avoid false alarms.
egrep -rl '^#!\s*%{_bindir}/env\s+python' --include=\*.py "%{buildroot}" |
    while read file; do
        sed -r '1s,^#!\s*%{_bindir}/env\s+python,#!%{__python},' -i "$file"
    done

# Eliminate Source Files; Those reading this spec file will already notice
# i've violated a million things; infact, this whole RPM should have been
# broken into 7 sepearate RPMs. But I just want to run GIMP, and i'm already
# going through the through the trouble of packaging it for myself and sharing
# it.  I encourage anyone to mend and take this further and do it easier.
# For now... it's a GIMP v2.8 all in one package.
rm -rf %{buildroot}%{_libdir}/pkgconfig
rm -rf %{buildroot}/usr/include

# We want to move all of our content into a virtual location so we don't
# obstruct existing data and jepordize our system integrity
mkdir -p  %{buildroot}%{install_root}
mv  %{buildroot}/usr %{buildroot}%{install_root}
mv  %{buildroot}/etc %{buildroot}%{install_root}

# More Cleanup
find %{buildroot} -name "*.py?" -delete
sed -e "s|%{buildroot}|%{install_root}|g" \
              %{buildroot}%{install_root}/etc/gtk-2.0/gtk.immodules

# Create some scripts to make life easy
mkdir -p %{buildroot}/usr/bin
cat <<'EOF' > %{buildroot}/usr/bin/gimp28
#!/bin/sh
export LD_LIBRARY_PATH=%{install_root}%{_libdir}
export GDK_PIXBUF_MODULEDIR=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders
export GDK_PIXBUF_MODULE_FILE=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders.cache
export GIMP2_DIRECTORY=.gimp-2.8
export GIMP2_DATADIR=%{install_root}/usr/share/gimp/2.0
export GIMP2_PLUGINDIR=%{install_root}%{_libdir}/gimp/2.0
export GIMP2_SYSCONFDIR=%{install_root}/etc/gimp/2.0
# support arguments as well for those who want to use them
%{install_root}/usr/bin/gimp "$@"
EOF
chmod 775 %{buildroot}/usr/bin/gimp28

cat <<'EOF' > %{buildroot}/usr/bin/gimp28-console
#!/bin/sh
export LD_LIBRARY_PATH=%{install_root}%{_libdir}
export GDK_PIXBUF_MODULEDIR=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders
export GDK_PIXBUF_MODULE_FILE=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders.cache
export GIMP2_DIRECTORY=.gimp-2.8
export GIMP2_DATADIR=%{install_root}/usr/share/gimp/2.0
export GIMP2_PLUGINDIR=%{install_root}%{_libdir}/gimp/2.0
export GIMP2_SYSCONFDIR=%{install_root}/etc/gimp/2.0
# support arguments as well for those who want to use them
%{install_root}/usr/bin/gimp-console "$@"
EOF
chmod 775 %{buildroot}/usr/bin/gimp28-console

cat <<'EOF' > %{buildroot}/usr/bin/gimp28tool
#!/bin/sh
export LD_LIBRARY_PATH=%{install_root}%{_libdir}
export GDK_PIXBUF_MODULEDIR=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders
export GDK_PIXBUF_MODULE_FILE=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders.cache
export GIMP2_DIRECTORY=.gimp-2.8
export GIMP2_DATADIR=%{install_root}/usr/share/gimp/2.0
export GIMP2_PLUGINDIR=%{install_root}%{_libdir}/gimp/2.0
export GIMP2_SYSCONFDIR=%{install_root}/etc/gimp/2.0
# support arguments as well for those who want to use them
%{install_root}/usr/bin/gimptool "$@"
EOF
chmod 775 %{buildroot}/usr/bin/gimp28tool

# Setup Gnome Menu Item
mkdir -p %{buildroot}/usr/share/applications
mv gimp28.desktop %{buildroot}/usr/share/applications

%clean
rm -rf %{buildroot}

%post
update-desktop-database &> /dev/null || :
# gdk-pixbuf updates
touch --no-create %{_datadir}/icons/hicolor &>/dev/null || :
export LD_LIBRARY_PATH=%{install_root}%{_libdir}
export GDK_PIXBUF_MODULEDIR=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders
export GDK_PIXBUF_MODULE_FILE=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders.cache
export PATH="%{install_root}/usr/bin:$PATH"
gdk-pixbuf-query-loaders-%{__isa_bits} --update-cache &>/dev/null || :

%postun
update-desktop-database &> /dev/null || :
if [ $1 -eq 0 ] ; then
    touch --no-create %{_datadir}/icons/hicolor &>/dev/null
    gtk-update-icon-cache %{_datadir}/icons/hicolor &>/dev/null || :

    # gdk-pixbuf updates
    export LD_LIBRARY_PATH=%{install_root}%{_libdir}
    export GDK_PIXBUF_MODULEDIR=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders
    export GDK_PIXBUF_MODULE_FILE=%{install_root}%{_libdir}/gdk-pixbuf-2.0/2.10.0/loaders.cache
    export PATH="%{install_root}/usr/bin:$PATH"
    gdk-pixbuf-query-loaders-%{__isa_bits} --update-cache &>/dev/null || :
fi

%posttrans
gtk-update-icon-cache %{_datadir}/icons/hicolor &>/dev/null || :

%files
/opt/gimp28/*
/usr/bin/gimp28
/usr/bin/gimp28-console
/usr/bin/gimp28tool
/usr/share/applications/gimp28.desktop

%changelog
* Fri Apr 25 2014 - Chris Caron <lead2gold@gmail.com> v2.8.10-1
- Upgraded base to mainstream v2.8.10
* Sat Oct 05 2013 - Chris Caron <lead2gold@gmail.com> v2.8.6-1
- Initial Packaging of GIMP v2.8.6 for CentOS/Redhat 6
